#include "L3G4200D_gyroscope.h"
#include "battery.h"

//Demo gyroscope driving to the Attitude indicator on the host
void demo_gyroscope(void)
{
	Init_gyroscope();							//Enable gyroscope
	init_timer3(0.01);							//100Hz
	while(1) {
		delay_ms(100);							//10 times per second
		printf("%4.3f,%4.3f,%4.3f\n",g_pitch,g_roll,g_yaw);
		if(RxReady()){
			NVIC_DisableIRQ(TIMER3_IRQn); 		//Disable timer interrupt
			return;
		}
	}
}

void demo_gyroscopeintegral(void)
{
	Init_gyroscope();							//Enable gyroscope
	Init_accelerometer();						//Enable accelerometer
	init_timer3(0.02);							//start integrating sensors
	while(1){
		delay_ms(100);							//10 times per second
		printf("%4.3f,%4.3f,%4.3f\n",g_pitch,g_roll,g_yaw);
		if(RxReady()){
			NVIC_DisableIRQ(TIMER3_IRQn); 		// Disable timer interrupt
			return;
		}
	}
}

//Enable the gyroscope. That is power up and enable each axis
//return 0 for success and -1 for failure
char Init_gyroscope(void)
{
	if(i2c0_read(L3G4200D,0x0f)!=0xd3){			//check if device is attached
		printf("L3G4200D not found.");
		return -1;
	}
	i2c0_write(L3G4200D, CTRL_REG1, (PD | XEN | YEN | ZEN | DR400 | BW110));	//enable
	i2c0_write(L3G4200D, CTRL_REG4, DPS500 );	//scale
	GyroSensitivity = (float) GyroSens500;
	Calibrate_gyroscope();
	return 0;
}

void print_Gyro(void)
{
	printf(" Gyroscope     X = %3d  ",PitchSpeed());
	printf("Y = %3d  Z = %3d\n",RollSpeed(),YawSpeed());
}

//The L3G4200D gyroscope must be calibrated to remove the zero rate error
//of the sensor. The process involves calculating the offset error when the
//expected rate is X=Y=Z=0 when the sensors is resting level
//on the ground.
//Steps:
//  1)Find median of sensor readings over 128 samples
//	2)calculate the offset value as the difference between the median
//	  and the expected 0 result
//	3)Export results to global variables
void Calibrate_gyroscope(void)
{
	int i;
	signed short X[128],Y[128],Z[128];

	for(i=0;i<128;i++){								//Get 128 reading
		delay_ms(1);
		X[i] = (i2c0_read(L3G4200D,OUT_X_L) | \
				i2c0_read(L3G4200D,OUT_X_H)<<8);
		Y[i] = (i2c0_read(L3G4200D,OUT_Y_L) | \
				i2c0_read(L3G4200D,OUT_Y_H)<<8);
		Z[i] = (i2c0_read(L3G4200D,OUT_Z_L) | \
				i2c0_read(L3G4200D,OUT_Z_H)<<8);
	}

	//Median 128 reading to determine starting value
	Xgoffset = 0-median(X, 128);
	Ygoffset = 0-median(Y, 128);
	Zgoffset = 0-median(Z, 128);
	debug1(" Gyroscope offsets X = %4d  ",Xgoffset);
	debug1("Y = %4d  Z = %4d\n",Ygoffset,Zgoffset);
	print_Gyro();
}

//This routine is private and should only be called by the Init_gyroscope()
//function. Timer3 is used as an interval timer to allow for a reliable and
//predictable dt sampling period.
//Timer3 maintains the g_dt sample interval that all of the sensor functions
//operate at. There are two sets of features managed in this interrupt. First
//the gyro sensor readings are integrate to determine the pitch and roll.
//Optionally the gyro values can be improved using the accelerometer readings
//and a complimentary filter. The second function managed by this code is the
//control loop callbacks which must run last since they are dependant on the
//sensor results established in step 1.
void init_timer3(float dt)
{
	g_dt = dt;
	LPC_SC->PCONP |= 1 << 23; 					// Power up Timer 3
	LPC_SC->PCLKSEL0 |= 1 << 2; 				// Clock for timer = CCLK
	LPC_TIM3->MR3 = (dt*1000000); 				// dt*1MHz match value
	LPC_TIM3->PR = 24;							// 72MHz/24/2 = 1MHz
	LPC_TIM3->MCR |= 1 << 9; 					// Interrupt on Match3
	LPC_TIM3->MCR |= 1 << 10; 					// Reset timer on Match3
	LPC_TIM3->TCR |= 1 << 10;					// Force Reset of Timer
	LPC_TIM3->TCR &= ~(1 << 10); 				// release reset on Timer
	NVIC_SetPriority(TIMER3_IRQn, 5);			// Set priority lower than I2C0
	NVIC_EnableIRQ(TIMER3_IRQn); 				// Enable timer interrupt
	LPC_TIM3->TCR |= 1 << 0; 					// Start timer
	g_dt_samples = 0;

	g_pitch = 0;								//reset globals to zero
	g_roll = 0;
	g_yaw = 0;
	g_filter_pitch = 0;
	g_filter_roll = 0;
	gyro_saturation = 0;
}

//Initialise the integration function by starting the timer interrupt.
//The sample frequency is used to initialise Timer3 to give a predictable
//dt interval. Note that the maximum sample rate supported by the L3G4200
//sensor is 100Hz so any attempt to set a faster rate than this is clamped
//to the hardwares sample rate. The rate could be up to 800Hz with a different
//initiallization.
void TIMER3_IRQHandler(void)
{
	void (*ptr)();
	g_dt_samples += g_dt;						//debug sample timer
    if((LPC_TIM3->IR & 1<<3)) { 				//if MR3 interrupt
    	ledr(1);								//debug trace for scope
  //  	led2(1);
    	g_pitch_speed = PitchSpeed();			//Global rates
    	g_roll_speed = RollSpeed();
    	g_yaw_speed = YawSpeed();
  //  	ledg(1);
    	ledr(0);
    	//integrate gyro to track the attitude angle
        g_pitch += (float)g_pitch_speed*g_dt;
        g_roll += (float)g_roll_speed*g_dt;
        g_yaw += (float)g_yaw_speed*g_dt;
        ledg(1);
        UpdateGlobalAngles();					//Update accelerometer globals
        ledg(0);
        //calculate complimentary filter results
        g_filter_pitch = Khpf*(g_filter_pitch+(float)g_pitch_speed*g_dt)\
        		       + Klpf*g_AngleX;

        g_filter_roll  = Khpf*(g_filter_roll+(float)g_roll_speed*g_dt)\
        		       + Klpf*g_AngleY;
  //      led2(0);
        LPC_TIM3->IR |= 1<<3; 					// Clear MR3 interrupt
    	ptr = PID_callback;
    	(*ptr)();								//call PID after sensor integral
 //   	led1(0);								//debug trace for scope
    }
}

//The roll rate measured in degrees per second. These values are offset
//corrected so that any initial offsets are cancelled.
signed short RollSpeed(void)
{
	signed short Xmeasured,Xcorrected,RollSpeed;
	Xmeasured =(i2c0_read(L3G4200D,OUT_X_L)|i2c0_read(L3G4200D,OUT_X_H)<<8);
	Xcorrected = (Xmeasured - Xgoffset);
	if(Xcorrected<0){
		//RollSpeed = Xcorrected*GyroSensitivity;500/32768
		RollSpeed = (Xcorrected+1)*500/32768;
	}else{
		RollSpeed = (Xcorrected-1)*500/32767;
	}
	return RollSpeed;
}

//The pitch rate measured in degrees per second. These values are offset
//corrected so that any initial offsets are cancelled.
signed short PitchSpeed(void)
{
	signed short Ymeasured,Ycorrected,PitchSpeed;
	Ymeasured =(i2c0_read(L3G4200D,OUT_Y_L)|i2c0_read(L3G4200D,OUT_Y_H)<<8);
	Ycorrected = Ymeasured - Ygoffset;
	PitchSpeed = Ycorrected*GyroSensitivity;
	return PitchSpeed;
}

//The yaw rate measured in degrees per second. These values are offset
//corrected so that any initial offsets are cancelled.
signed short YawSpeed(void)
{
	signed short Zmeasured,Zcorrected,YawSpeed;
	Zmeasured =(i2c0_read(L3G4200D,OUT_Z_L)|i2c0_read(L3G4200D,OUT_Z_H)<<8);
	Zcorrected = Zmeasured - Zgoffset;
	YawSpeed = Zcorrected*GyroSensitivity;
	return YawSpeed;
}
