#include "MMA7455_accelerometer.h"

//Demo accelerometer driving to the Attitude indicator on the host
//To reduce the noise in the accelerometer output a weighted average is used
//to filter the print results. This effectively only shows the long term
//direction of gravity since noise is filtered out. The down side is
//that the weighted average takes time to respond and introduces a lag
//in the reported results. It makes for a better displayed result but the
//instantaneous angle is rather out of date due to the phase lag.
void demo_accelerometer(void)
{
	double Rx,Ry,Rz;
	Init_accelerometer();							//Enable accelerometer
	while(1) {
		if(RxReady()){return;}
		delay_ms(100);								//10 times per second
		UpdateGlobalAngles();
		//Weighted average
			Rx = (g_AngleX + 3.0f*RxPrevious)/4.0f;
			Ry = (g_AngleY + 3.0f*RyPrevious)/4.0f;
			Rz = (g_AngleZ + 3.0f*RzPrevious)/4.0f;

			RxPrevious = Rx;
			RyPrevious = Ry;
			RzPrevious = Rz;
		printf("%4.3f,%4.3f,%4.3f\n",Rx,Ry,Rz);
	}
}

//Enable the accelerometer with 2g sensitivity
//return 0 for success and -1 for failure
char Init_accelerometer(void)
{
	if(i2c0_read(MMA7455, I2CAD)!=0x1d){	//check if device is attached
		printf("MMA7455 not found.");
		return -1;
	}
	i2c0_write(MMA7455, MODE_CONTROL, (LVL2G | MEASURE));	//enable
	Calibrate_accelerometer();
	return 0;
}

//The MMA7455 accelerometer has an internal offset register that can be used
//to correct sensor readings. However we use a software routine for consistency
//with the other sensors. The process involves zeroing the X=Y=0 axis
//and setting the Z axis to full scale 1g when the sensors is resting level
//on the ground.
//Steps:
//  1)sensor median readings over 128 samples
//	2)calculate the offset X,Y = 0-median, Z = 63-median

void Calibrate_accelerometer(void)
{
	int i;
	signed short Xraw[128],Yraw[128],Zraw[128];

	for(i=0;i<128;i++){							//Get 128 raw reading
		delay_ms(1);
		Xraw[i] = (short)i2c0_read(MMA7455,XOUT8);
		Yraw[i] = (short)i2c0_read(MMA7455,YOUT8);
		Zraw[i] = (short)i2c0_read(MMA7455,ZOUT8);
	}
	print_Accelerometer();
	//Median 128 raw reading to determine starting value
	Xoffset= 0-median(Xraw, 128);				//expect 0
	Yoffset= 0-median(Yraw, 128);				//expect 0
	Zoffset= 63-(median(Zraw, 128));			//expect 63 full scale 1G
	debug1("\n Accelerometer offset values X = %3d  Y = %3d  Z = %3d\n",\
			Xoffset,Yoffset,Zoffset);
	print_Accelerometer();
}

//Measure the X axis results with the offset error removed. The offset
//error is determined by running the calibrate function.
signed char Xmeasured(void)
{
	signed char result;
	result = i2c0_read(MMA7455,XOUT8);
	result +=  Xoffset;
	return result;
}

//Measure the Y axis results with the offset error removed. The offset
//error is determined by running the calibrate function.
signed char Ymeasured(void)
{
	signed char result;
	result = i2c0_read(MMA7455,YOUT8);
	result += Yoffset;
	return result;
}

//Measure the Z axis results with the offset error removed. The offset
//error is determined by running the calibrate function.
signed char Zmeasured(void)
{
	signed char result;
	result = i2c0_read(MMA7455,ZOUT8);
	result += Zoffset;
	return result;
}

//The accelerometer is used to measure the average direction of gravity
//which is defined arbitrarily as 1g. Because the acceleration due to
//gravity effects all three axis in the same way; we can correlated the
//measurements for better accuracy. This is done by scaling the results
//and normalizing them to a unit value of 1g before determining the
//angle. The three acceleration angles relative to gravity are updated
//as global variables g_AngleX, g_AngleY, g_AngleZ.
void UpdateGlobalAngles(void)
{
double Rx,Ry,Rz,Racc;
//led2(1);
	Rx = (double)Xmeasured()/(double)FullScale1G;
//led2(0);
	Ry = (double)Ymeasured()/(double)FullScale1G;
	Rz = (double)Zmeasured()/(double)FullScale1G;
//Magnitude of acceleration
	Racc = (double)sqrt((double)(Rx*Rx+Ry*Ry+Rz*Rz));	//absolute value
//Normalise
	Rx = ((double)Rx/(double)Racc);
	Ry = ((double)Ry/(double)Racc);
	Rz = ((double)Rz/(double)Racc);

//Convert to degrees
	g_AngleX = (double)asinf(Rx)*Rad2Degrees;
	g_AngleY = -1.0f*(double)asinf(Ry)*Rad2Degrees;		//Negative scaling
	g_AngleZ = (double)asinf(Rz)*Rad2Degrees;
}
/*
float Xaccelerometer(void)
{
	float result;
	result = (float)Xmeasured()/(float)FullScale1G;
	if(result > 1){result = 1;};
	if(result < -1){result = -1;};
	result = (float)asinf((float)result)*Rad2Degrees;
	return result;
}

float Yaccelerometer(void)
{
	float result;
	result = (float)Ymeasured()/(float)FullScale1G;
	if(result > 1){result = 1;};
	if(result < -1){result = -1;};
	result = (float)asinf((float)result)*Rad2Degrees;
	return result;
}
float Zaccelerometer(void)
{
	float result;
	result = (float)Zmeasured()/(float)FullScale1G;
	if(result > 1){result = 1;};
	if(result < -1){result = -1;};
	result = (float)asinf((float)result)*Rad2Degrees;
	return result;
}*/

void print_Accelerometer(void)
{
	printf(" Accelerometer X = %3d  Y = %3d  Z = %3d\n",\
			Xmeasured(),Ymeasured(),Zmeasured());
}

