/****************************************************************************
 *   $Id:: i2c.c 5865 2010-12-08 21:42:21Z usb00423                         $
 *   Project: NXP LPC17xx I2C example
 *
 *   Description:
 *     This file contains I2C code example which include I2C initialization, 
 *     I2C interrupt handler, and APIs for I2C access.
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/
#include "lpc17xx.h"
#include "../../RDB1768cmsis2_usbstack/inc/type.h"
#include "i2c.h"
#include <stdio.h>
#include <string.h>

volatile uint32_t I2CMasterState[I2C_PORT_NUM] = {I2C_IDLE,I2C_IDLE};
volatile uint32_t timeout[I2C_PORT_NUM] = {0, 0};

volatile uint8_t I2CMasterBuffer[I2C_PORT_NUM][BUFSIZE];
volatile uint8_t I2CSlaveBuffer[I2C_PORT_NUM][BUFSIZE];
volatile uint32_t I2CCount[I2C_PORT_NUM] = {0, 0};
volatile uint32_t I2CReadLength[I2C_PORT_NUM];
volatile uint32_t I2CWriteLength[I2C_PORT_NUM];

volatile uint32_t RdIndex0 = 0, RdIndex1 = 0;
volatile uint32_t WrIndex0 = 0, WrIndex1 = 0;

/* 
From device to device, the I2C communication protocol may vary, 
in the example below, the protocol uses repeated start to read data from or 
write to the device:
For master read: the sequence is: STA,Addr(W),offset,RE-STA,Addr(r),data...STO 
for master write: the sequence is: STA,Addr(W),offset,RE-STA,Addr(w),data...STO
Thus, in state 8, the address is always WRITE. in state 10, the address could 
be READ or WRITE depending on the I2C command.
*/   

/****************************************************************************
** Function name:		I2C_IRQHandler
**
** Descriptions:		I2C interrupt handler, deal with master mode only.
**						No support for multi-master
** parameters:			None
** Returned value:		None
** 
****************************************************************************/
void I2C0_IRQHandler(void) 
{
  uint8_t StatValue;

  timeout[0] = 0;
  StatValue = LPC_I2C0->I2STAT;
  switch ( StatValue )
  {
	case 0x08:								//Start condition is issued.
printf("Start condition is issued.\n");
	WrIndex0 = 0;
	LPC_I2C0->I2DAT = I2CMasterBuffer[0][WrIndex0++];
	LPC_I2C0->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x10:								//repeated started is issued
printf("repeated started is issued.\n");
	RdIndex0 = 0;
	// Send SLA with R bit set,
	LPC_I2C0->I2DAT = I2CMasterBuffer[0][WrIndex0++];
	LPC_I2C0->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x18:								// Regardless, it's a ACK
printf("0x18 \n");
	if ( I2CWriteLength[0] == 1 ){
	  LPC_I2C0->I2CONSET = I2CONSET_STO; 	// Set Stop flag
	  I2CMasterState[0] = I2C_NO_DATA;
	}else{
	  LPC_I2C0->I2DAT = I2CMasterBuffer[0][WrIndex0++];
	}
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x28:	// Data byte has been transmitted, regardless ACK or NACK
printf("0x28 Data byte has been transmitted.\n");
	if ( WrIndex0 < I2CWriteLength[0] ){
	  LPC_I2C0->I2DAT = I2CMasterBuffer[0][WrIndex0++]; //should be last one
	}else{
	  if ( I2CReadLength[0] != 0 ){
		LPC_I2C0->I2CONSET = I2CONSET_STA;	// Set Repeated-start flag
	  }else{
		LPC_I2C0->I2CONSET = I2CONSET_STO;  // Set Stop flag
		I2CMasterState[0] = I2C_OK;
	  }
	}
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x30:
printf("0x30 Set Stop flag.\n");
	LPC_I2C0->I2CONSET = I2CONSET_STO;      // Set Stop flag
	I2CMasterState[0] = I2C_NACK_ON_DATA;
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x40:								//Master Receive, SLA_R sent
printf("Master Receive, SLA_R sent.\n");
	if ( (RdIndex0 + 1) < I2CReadLength[0] ){
	  // Will go to State 0x50
	  LPC_I2C0->I2CONSET = I2CONSET_AA;		//assert ACK after data is received
	  printf("ACK\n");
	}else{
	  // Will go to State 0x58
	  LPC_I2C0->I2CONCLR = I2CONCLR_AAC;	//NACK after data is received
	  printf("NACK\n");
	}
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x50:	// Data byte has been received, regardless following ACK or NACK
printf("0x50 Data byte has been received\n");
	I2CSlaveBuffer[0][RdIndex0++] = LPC_I2C0->I2DAT;
	if ( (RdIndex0 + 1) < I2CReadLength[0] ){
	  LPC_I2C0->I2CONSET = I2CONSET_AA;		// ACK after data is received
	  printf("ACK\n");
	}else{
	  LPC_I2C0->I2CONCLR = I2CONCLR_AAC;	//NACK on last byte
	  printf("NACK\n");
	}
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x58:
printf("0x58 Set Stop flag.\n");
	I2CSlaveBuffer[0][RdIndex0++] = LPC_I2C0->I2DAT;
	I2CMasterState[0] = I2C_OK;
	LPC_I2C0->I2CONSET = I2CONSET_STO;		// Set Stop flag
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;		// Clear SI flag
	break;

	case 0x20:								// regardless, it's a NACK
	case 0x48:
printf("0x20 0x48 Set Stop flag \n");
	LPC_I2C0->I2CONSET = I2CONSET_STO;      // Set Stop flag
	I2CMasterState[0] = I2C_NACK_ON_ADDRESS;
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x38:								// Arbitration lost
	default:
printf("Arbitration lost.\n");
	I2CMasterState[0] = I2C_ARBITRATION_LOST;
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
  }
  return;
}

/****************************************************************************
** Function name:		I2C_IRQHandler
**
** Descriptions:		I2C interrupt handler, deal with master mode only.
**						No support for multi-master
** parameters:			None
** Returned value:		None
** 
****************************************************************************/
void I2C1_IRQHandler(void) 
{
  uint8_t StatValue;

  timeout[1] = 0;
  StatValue = LPC_I2C1->I2STAT;
  switch ( StatValue )
  {
	case 0x08:								//Start condition is issued.
	WrIndex1 = 0;
	LPC_I2C1->I2DAT = I2CMasterBuffer[1][WrIndex1++];
	LPC_I2C1->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x10:								//repeated started is issued
	RdIndex1 = 0;
	// Send SLA with R bit set,
	LPC_I2C1->I2DAT = I2CMasterBuffer[1][WrIndex1++];
	LPC_I2C1->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x18:								// Regardless, it's a ACK
	if ( I2CWriteLength[1] == 1 ){
	  LPC_I2C1->I2CONSET = I2CONSET_STO;    // Set Stop flag
	  I2CMasterState[1] = I2C_NO_DATA;
	}else{
	  LPC_I2C1->I2DAT = I2CMasterBuffer[1][WrIndex1++];
	}
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x28:	// Data byte has been transmitted, regardless ACK or NACK
	if ( WrIndex1 < I2CWriteLength[1] ){
	  LPC_I2C1->I2DAT = I2CMasterBuffer[1][WrIndex1++]; //should be last one
	}else{
	  if ( I2CReadLength[1] != 0 ){
		LPC_I2C1->I2CONSET = I2CONSET_STA;	// Set Repeated-start flag
	  }else{
		LPC_I2C1->I2CONSET = I2CONSET_STO;  // Set Stop flag
		I2CMasterState[1] = I2C_OK;
	  }
	}
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x30:
	LPC_I2C1->I2CONSET = I2CONSET_STO;      // Set Stop flag
	I2CMasterState[1] = I2C_NACK_ON_DATA;
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x40:								//Master Receive, SLA_R sent
	if ( (RdIndex1 + 1) < I2CReadLength[1] ){
											//Will go to State 0x50
	  LPC_I2C1->I2CONSET = I2CONSET_AA;		//ACK after data is received
	}else{
											// Will go to State 0x58
	  LPC_I2C1->I2CONCLR = I2CONCLR_AAC;	//NACK after data is received
	}
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x50:	// Data byte has been received, regardless following ACK or NACK
	I2CSlaveBuffer[1][RdIndex1++] = LPC_I2C1->I2DAT;
	if ( (RdIndex1 + 1) < I2CReadLength[1] ){
	  LPC_I2C1->I2CONSET = I2CONSET_AA;		//ACK after data is received
	}else{
	  LPC_I2C1->I2CONCLR = I2CONCLR_AAC;	//NACK on last byte
	}
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x58:
	I2CSlaveBuffer[1][RdIndex1++] = LPC_I2C1->I2DAT;
	I2CMasterState[1] = I2C_OK;
	LPC_I2C1->I2CONSET = I2CONSET_STO;		// Set Stop flag
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;		// Clear SI flag
	break;

	case 0x20:								// regardless, it's a NACK
	case 0x48:
	LPC_I2C1->I2CONSET = I2CONSET_STO;      // Set Stop flag
	I2CMasterState[1] = I2C_NACK_ON_ADDRESS;
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x38:								// Arbitration lost
	default:
	I2CMasterState[1] = I2C_ARBITRATION_LOST;
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
  }
  return;
}

/****************************************************************************
** Function name:		I2CStart
**
** Descriptions:		Create I2C start condition, a timeout
**						value is set if the I2C never gets started,
**						and timed out. It's a fatal error.
**
** parameters:			None
** Returned value:		true or false, return false if timed out
** 
*****************************************************************************/
uint32_t I2CStart( uint32_t portNum )
{
  uint32_t retVal = FALSE;
 
  timeout[portNum] = 0;
  // Issue a start condition
  LPC_I2C[portNum]->I2CONSET = I2CONSET_STA;// Set Start flag
    
  // Wait until START transmitted
  while( 1 ){
	if ( I2CMasterState[portNum] == I2C_STARTED ){
	  retVal = TRUE;
	  break;	
	}
	if ( timeout[portNum] >= MAX_TIMEOUT ){
	  retVal = FALSE;
	  break;
	}
	timeout[portNum]++;
  }
  return( retVal );
}

/****************************************************************************
** Function name:		I2CStop
**
** Descriptions:		Set the I2C stop condition, if the routine
**						never exit, it's a fatal bus error.
**
** parameters:			None
** Returned value:		true or never return
** 
****************************************************************************/
uint32_t I2CStop( uint32_t portNum )
{
  LPC_I2C[portNum]->I2CONSET = I2CONSET_STO;// Set Stop flag
  LPC_I2C[portNum]->I2CONCLR = I2CONCLR_SIC;// Clear SI flag
            
  //Wait for STOP detected
  while( LPC_I2C[portNum]->I2CONSET & I2CONSET_STO );
  return TRUE;
}

/****************************************************************************
** Function name:		I2CInit
**
** Descriptions:		Initialise I2C controller as a master
**
** parameters:			None
** Returned value:		None
** 
****************************************************************************/
void I2C0Init( void ) 
{
  LPC_SC->PCONP |= (1 << 7);

  // set PIO0.27 and PIO0.28 to I2C0 SDA and SCL
  // function to 01 on both SDA and SCL.
  LPC_PINCON->PINSEL1 &= ~((0x03<<22)|(0x03<<24));
  LPC_PINCON->PINSEL1 |= ((0x01<<22)|(0x01<<24));	
 
  // Clear flags
  LPC_I2C0->I2CONCLR=I2CONCLR_AAC|I2CONCLR_SIC|I2CONCLR_STAC|I2CONCLR_I2ENC;

  // Reset registers
  LPC_PINCON->I2CPADCFG &= ~((0x1<<0)|(0x1<<2));
  LPC_I2C0->I2SCLL   = I2SCLL_SCLL;
  LPC_I2C0->I2SCLH   = I2SCLH_SCLH;

  // Install interrupt handler
  NVIC_EnableIRQ(I2C0_IRQn);

  LPC_I2C0->I2CONSET = I2CONSET_I2EN;
  return;
}

/****************************************************************************
** Function name:		I2C1Init
**
** Descriptions:		Initialize I2C controller as a master
**
** parameters:			None
** Returned value:		None
** 
****************************************************************************/
void I2C1Init( void ) 
{
  LPC_SC->PCONP |= (1 << 19);

  // set PIO0.19 and PIO0.20 to I2C1 SDA and SCL
  // function to 11 on both SDA and SCL.
  LPC_PINCON->PINSEL1 &= ~((0x3<<6)|(0x3<<8));
  LPC_PINCON->PINSEL1 |= ((0x3<<6)|(0x3<<8));
  LPC_PINCON->PINMODE1 &= ~((0x3<<6)|(0x3<<8));
  LPC_PINCON->PINMODE1 |= ((0x2<<6)|(0x2<<8));	// No pull-up no pull-down
  LPC_PINCON->PINMODE_OD0 |= ((0x1<<19)|(0x1<<20));	
 
  // Clear flags
  LPC_I2C1->I2CONCLR=I2CONCLR_AAC|I2CONCLR_SIC|I2CONCLR_STAC|I2CONCLR_I2ENC;

  // Reset registers
  LPC_I2C1->I2SCLL   = I2SCLL_SCLL;
  LPC_I2C1->I2SCLH   = I2SCLH_SCLH;

  // Install interrupt handler
  NVIC_EnableIRQ(I2C1_IRQn);

  LPC_I2C1->I2CONSET = I2CONSET_I2EN;
  return;
}

/****************************************************************************
** Function name:		I2CEngine
**
** Descriptions:		The routine to complete an I2C transaction
**						from start to stop. All the intermediate
**						steps are handled in the interrupt handler.
**						Before this routine is called, the read
**						length, write length, I2C master buffer,
**						and I2C command fields need to be filled.
**
** parameters:			I2C port number
** Returned value:		master state of current I2C port.
** 
****************************************************************************/
uint32_t I2CEngine( uint32_t portNum ) 
{
  LPC_I2C[portNum]->I2CONSET = I2CONSET_STA;	// Issue a start condition

  I2CMasterState[portNum] = I2C_BUSY;	

  while ( I2CMasterState[portNum] == I2C_BUSY )
  {
	if ( timeout[portNum] >= MAX_TIMEOUT )
	{
	  I2CMasterState[portNum] = I2C_TIME_OUT;
	  break;
	}
	timeout[portNum]++;
  }
  LPC_I2C[portNum]->I2CONCLR = I2CONCLR_STAC;

  return ( I2CMasterState[portNum] );
}

void PrintI2CState(unsigned char state)
{
	switch (state){
	case I2C_IDLE:
		printf("I2C_IDLE");
		break;
	case I2C_STARTED:
		printf("I2C_STARTED");
		break;
	case I2C_RESTARTED:
		printf("I2C_RESTARTED");
		break;
	case I2C_REPEATED_START:
		printf("I2C_REPEATED_START");
		break;
	case DATA_ACK :
		printf("DATA_ACK");
		break;
	case DATA_NACK :
		printf("DATA_NACK");
		break;
	case I2C_BUSY :
		printf("I2C_BUSY");
		break;
	case I2C_NO_DATA :
		printf("I2C_NO_DATA");
		break;
	case I2C_NACK_ON_ADDRESS :
		printf("I2C_NACK_ON_ADDRESS");
		break;
	case I2C_NACK_ON_DATA :
		printf("I2C_NACK_ON_DATA");
		break;
	case I2C_ARBITRATION_LOST :
		printf("I2C_ARBITRATION_LOST");
		break;
	case I2C_TIME_OUT :
		printf("I2C_TIME_OUT");
		break;
	case I2C_OK :
		printf("I2C_OK");
		return;
		break;
	default:
		printf("Unrecognized command");
		break;
	};
	printf("\n");
}

void I2CWrite(int port, char * command)
{
	int i;
	  I2CWriteLength[port] = command[0];
	  I2CReadLength[port] = 0;
	  for(i=1;i<command[0]+1;i++){
		  I2CMasterBuffer[port][i-1] = command[i];
	  }
	  PrintI2CState(I2CEngine(port));
}

void I2CRead(int port, char * command)
{
	int i;
	  I2CWriteLength[port] = command[0];
	  I2CReadLength[port] = command[1];
	  for(i=2;i<command[0]+3;i++){
		  I2CMasterBuffer[port][i-2] = command[i];
	  }
	  PrintI2CState(I2CEngine(port));
}

void FindI2CDevices(int port)
{
	int i;
	char temp;
	  printf("\r\nSearching I2C%c for devices.",port);
	  I2CWriteLength[port] = 0x01;		//
	  I2CReadLength[port] = 0x00;
	  I2CMasterBuffer[port][1] = 0x00;	//register 0
	  for(i=0;i<256;i++){
		  I2CMasterBuffer[port][0] = i;
		  temp = I2CEngine(port);
	  	  if( temp == I2C_OK){
	  		  printf("\r\nFound device at 0x%x",i);
	  	  }
	  }
}


void PrintMasterBuffer(int port, unsigned char length)
{
	int i;
	for(i=0;i<length;i++){
		printf("0x%x ",I2CMasterBuffer[port][i]);
	}
	printf("\r\n");
}

void PrintSlaveBuffer(int port, unsigned char length)
{
	int i;
	for(i=0;i<length;i++){
		printf("0x%x ",I2CSlaveBuffer[port][i]);
	}
	printf("\r\n");
}

//Writes a single byte of data to an I2C device
// device = I2C write address
// reg = the device register
// data = the data value to be written
void i2c0_write(char device, char reg, char data)
{
	I2CWriteLength[0] = 3;			//write one byte
	I2CReadLength[0] = 0;			//read nothing
	I2CMasterBuffer[0][0] = device;	//device address
	I2CMasterBuffer[0][1] = reg;	//device register
	I2CMasterBuffer[0][2] = data;	//device register
	I2CEngine(0);
}
void i2c1_write(char device, char reg, char data)
{
	I2CWriteLength[1] = 3;			//write one byte
	I2CReadLength[1] = 0;			//read nothing
	I2CMasterBuffer[1][0] = device;	//device address
	I2CMasterBuffer[1][1] = reg;	//device register
	I2CMasterBuffer[1][2] = data;	//device register
	I2CEngine(1);
}

//Read data from an I2C device and returns 8 bits
// device = I2C write address
// reg = the device register to be read
char i2c0_read(char device, char reg)
{
	char state;
	I2CWriteLength[0] = 2;			//write one byte
	I2CReadLength[0] = 1;			//read nothing
	I2CMasterBuffer[0][0] = device;	//device address
	I2CMasterBuffer[0][1] = reg;	//device register
	I2CMasterBuffer[0][2] = device | 0x01;	//device read
	if((state =I2CEngine(0)) != I2C_OK){
		printf("lost sync with sensor\n");
		PrintI2CState(state);
	};
	return I2CSlaveBuffer[0][0];
}
char i2c1_read(char device, char reg)
{
	char state;
	I2CWriteLength[1] = 2;			//write one byte
	I2CReadLength[1] = 1;			//read nothing
	I2CMasterBuffer[1][0] = device;	//device address
	I2CMasterBuffer[1][1] = reg;	//device register
	I2CMasterBuffer[1][2] = device | 0x01;	//device read
	if((state =I2CEngine(1)) != I2C_OK){
		printf("lost sync with sensor\n");
		PrintI2CState(state);
	};
	return I2CSlaveBuffer[1][0];
}
