#include "usb_cdc.h"
#include "uart.h"

static TLineCoding LineCoding = {115200, 0, 0, 8};

U8 txdata[VCOM_FIFO_SIZE];
U8 rxdata[VCOM_FIFO_SIZE];

fifo_t txfifo;
fifo_t rxfifo;

void CDC_Init(void)
{
	USBInit();										// initialise stack
	USBRegisterDescriptors(abDescriptors);			// register descriptors

// register class request handler
	USBRegisterRequestHandler(REQTYPE_TYPE_CLASS, HandleClassRequest, abClassReqData);

// register end point handlers
	USBHwRegisterEPIntHandler(INT_IN_EP, NULL);
	USBHwRegisterEPIntHandler(BULK_IN_EP, BulkIn);
	USBHwRegisterEPIntHandler(BULK_OUT_EP, BulkOut);
	USBHwRegisterFrameHandler(USBFrameHandler);		//register frame handler
	USBHwNakIntEnable(INACK_BI);					//enable bulk-in int on NAKs
	VCOM_init();
	NVIC_EnableIRQ(USB_IRQn); 						//enable_USB_interrupts();
	USBHwConnect(TRUE);								// connect to bus
}

//	Local function to handle incoming bulk data
//	 [in] bEP
//	 [in] bEPStatus
void BulkOut(U8 bEP, U8 bEPStatus)
{
	int i, iLen;

	if (fifo_free(&rxfifo) < MAX_PACKET_SIZE) {
		return;										// may not fit into fifo
	}

	// get data from USB into intermediate buffer
	iLen = USBHwEPRead(bEP, abBulkBuf, sizeof(abBulkBuf));
	for (i = 0; i < iLen; i++) {
		if (!fifo_put(&rxfifo, abBulkBuf[i])) {		// put into FIFO
			ASSERT(FALSE);							// overflow... :(
			break;
		}
	}
}

//	Local function to handle outgoing bulk data
//	[in] bEP
//	[in] bEPStatus
void BulkIn(U8 bEP, U8 bEPStatus)
{
	int i, iLen;

	if (fifo_avail(&txfifo) == 0) {
		// no more data, disable further NAK interrupts until next USB frame
		USBHwNakIntEnable(0);
		return;
	}

	// get bytes from transmit FIFO into intermediate buffer
	for (i = 0; i < MAX_PACKET_SIZE; i++) {
		if (!fifo_get(&txfifo, &abBulkBuf[i])) {
			break;
		}
	}
	iLen = i;

	if (iLen > 0) {									// send over USB
		USBHwEPWrite(bEP, abBulkBuf, iLen);
	}
}

//
//	Local function to handle the USB-CDC class requests
//
//	@param [in] pSetup
//	@param [out] piLen
//	@param [out] ppbData
//
BOOL HandleClassRequest(TSetupPacket *pSetup, int *piLen, U8 **ppbData)
{
	switch (pSetup->bRequest) {

	case SET_LINE_CODING:
		memcpy((U8 *)&LineCoding, *ppbData, 7);
		*piLen = 7;
		break;

	case GET_LINE_CODING:
		*ppbData = (U8 *)&LineCoding;
		*piLen = 7;
		break;

	case SET_CONTROL_LINE_STATE:					// bit0 = DTR, bit = RTS
		break;

	default:
		return FALSE;
	}
	return TRUE;
}

//	Initialises the VCOM port.
//	Call this function before using VCOM_putchar or VCOM_getchar
void VCOM_init(void)
{
	fifo_init(&txfifo, txdata);
	fifo_init(&rxfifo, rxdata);
}

//	Writes one character (c) to VCOM port
//	returns character written, or EOF if character could not be written
int USB_putchar(int c)
{
	if(c == '\n'){
		fifo_put(&txfifo, '\r');
	}
	return fifo_put(&txfifo, c) ? c : EOF;
}

//	Reads one character from VCOM port
//	returns character read, or EOF if character could not be read
int __USB_getchar(void)
{
	U8 c;
	return fifo_get(&rxfifo, &c) ? c : EOF;
}

char USB_RxReady(void)
{
	if(!fifo_avail(&rxfifo)){	// check if FIFO has data
		return FALSE;
	}
	return TRUE;
}

unsigned char USB_getchar(void)
{
	char c;
	c = __USB_getchar();
	while(c == 0xff){c = __USB_getchar();}
	return c;
}

//	Interrupt handler
void USB_IRQHandler(void)
{
	USBHwISR();
}

void USBFrameHandler(U16 wFrame)
{
	if (fifo_avail(&txfifo) > 0) {
		// data available, enable NAK interrupt on bulk in
		USBHwNakIntEnable(INACK_BI);
	}
}

//Declare the functions that are to be linked to the printf command
void Init_Com_Drivers(void * putchar, unsigned char (*getchar)(),char (*rx_ready)())
{
	putchar_callback = putchar;
	getchar_callback = getchar;
	rx_ready_callback = rx_ready;
}

//Select one of UART1,UART3 or CDC as the stdin and stdout function
//used by printf. This function also initiallizes the base drivers
//for the selected function. This includes setting the baudrate.
void Set_printf_IO(char device,int baudrate)
{
	switch (device){
	case UART1:
		UART1_Init(baudrate);
		Init_Com_Drivers(&putchar1,&getchar1,&rx_ready1);
		break;
	case UART3:
		UART3_Init(baudrate);
		Init_Com_Drivers(&putchar3,&getchar3,&rx_ready3);
		break;
	case CDC:
		CDC_Init();
		Init_Com_Drivers(&USB_putchar,&USB_getchar,&USB_RxReady);
		break;
	};
}

char RxReady(void)
{
	char (*ptr)();
	ptr = rx_ready_callback;
	return (*ptr)(); 								// device receive status
}

// Called by bottom level of printf routine within RedLib C library to write
// a character. This version writes the character to the putchar_callback().
int __sys_write(int iFileHandle, char *pcBuffer, int iLength)
{
	unsigned int i;
	void (*ptr)();
	for (i = 0; i<iLength; i++)
	{
		ptr = putchar_callback;
		(*ptr)(pcBuffer[i]); 						// print each character
	}
	return iLength;
}

// Called by bottom level of scanf routine within RedLib C library to read
// a character. This version writes the character to the getchar_callback().
int __sys_readc(void)
{
	char c;
	unsigned char (*ptr)();
	ptr = getchar_callback;
	while((c = (*ptr)()) != EOF){};
	return (int)c;
}

void __sys_writec(char data)
{
	char (*ptr)();
	ptr = putchar_callback;
	(*ptr)(data);
}

void put_char(char data)
{
	char (*ptr)();
	ptr = putchar_callback;
	(*ptr)(data);
}

char get_char(void)
{
	unsigned char (*ptr)();
	ptr = getchar_callback;
	while(!RxReady()){};
	return (*ptr)();
}
