#include "utility.h"
//Accurate millisecond timed delays independent of clock rate
//Uses SysTick interrupt to generate timing
//This routine is blocking until the timeout occurs. Of course
//interrupts running at a higher priority will still run.
void delay_ms(int interval)
{
  int start = current_time;
  while (current_time - start < interval);
}

//
// The following code is public domain.
// Algorithm by Torben Mogensen, implementation by N. Devillard.
//The median is the middle value in a range of numbers. The median
//is similar to an average but it tends to eliminate the
//influence of out lieing data. That is noise tends to be
//eliminated.
signed short median(signed short m[], int n)
{
	int i, less, greater, equal;
	int min, max, guess, maxltguess, mingtguess;
	min = max = m[0] ;
	for (i=1 ; i<n ; i++) {
		if (m[i]<min) min=m[i];
		if (m[i]>max) max=m[i];
	}
	while (1) {
		guess = (min+max)/2;
		less = 0; greater = 0; equal = 0;
		maxltguess = min ;
		mingtguess = max ;
		for (i=0; i<n; i++) {
			if (m[i]<guess) {
				less++;
				if (m[i]>maxltguess) maxltguess = m[i] ;
			} else if (m[i]>guess) {
				greater++;
				if (m[i]<mingtguess) mingtguess = m[i] ;
			} else equal++;
		}
		if (less <= (n+1)/2 && greater <= (n+1)/2) break ;
		else if (less>greater) max = maxltguess ;
		else min = mingtguess;
	}
	if (less >= (n+1)/2) return maxltguess;
	else if (less+equal >= (n+1)/2) return guess;
	else return mingtguess;
}

//Set a port pin low. This routine set the I/O direction to
//output and sets the pin state low.
void output_low(int port, int bit)
{
	if(port == 0){							//port0
		LPC_GPIO0->FIOMASK  = 0;			//clear mask
		LPC_GPIO0->FIODIR |= (1<<bit); 		//set bit to output
		LPC_GPIO0->FIOCLR = (1<<bit); 		//set bit low
	}
	if(port == 1){							//port1
		LPC_GPIO1->FIOMASK  = 0;			//clear mask
		LPC_GPIO1->FIODIR |= (1<<bit); 		//set bit to output
		LPC_GPIO1->FIOCLR = (1<<bit); 		//set bit low
	}
	if(port == 2){							//port2
		LPC_GPIO2->FIOMASK  = 0;			//clear mask
		LPC_GPIO2->FIODIR |= (1<<bit); 		//set bit to output
		LPC_GPIO2->FIOCLR = (1<<bit); 		//set bit to low
	}
	if(port == 3){							//port3
		LPC_GPIO3->FIOMASK  = 0;			//clear mask
		LPC_GPIO3->FIODIR |= (1<<bit); 		//set bit to output
		LPC_GPIO3->FIOCLR = (1<<bit); 		//set bit to low
	}
}

//Set a port pin high. This routine set the I/O direction to
//output and sets the pin state high.
void output_high(int port, int bit)
{
	if(port == 0){							//port0
		LPC_GPIO0->FIOMASK =0;				//clear mask
		LPC_GPIO0->FIODIR  |= (1<<bit); 	//set bit to output
		LPC_GPIO0->FIOSET  = (1<<bit); 		//set bit to high
	}
	if(port == 1){							//port1
		LPC_GPIO1->FIOMASK =0;				//clear mask
		LPC_GPIO1->FIODIR  |= (1<<bit); 	//set bit to output
		LPC_GPIO1->FIOSET  = (1<<bit);  	//set bit to high
	}
	if(port == 2){							//port2
		LPC_GPIO2->FIOMASK =0;				//clear mask
		LPC_GPIO2->FIODIR  |= (1<<bit); 	//set bit to output
		LPC_GPIO2->FIOSET  = (1<<bit);  	//set bit to high
	}
	if(port == 3){							//port3
		LPC_GPIO3->FIOMASK =0;				//clear mask
		LPC_GPIO3->FIODIR  |= (1<<bit); 	//set bit to output
		LPC_GPIO3->FIOSET  = (1<<bit);  	//set bit to high
	}
}

//Set a port pin to an input. This routine set the I/O direction to
//input and returns a value of 0 or 1 to indicate the pin state.
int input(int port, int bit)
{

	if(port == 0){							//port0
		LPC_GPIO0->FIOMASK =~(1<<bit);		//set mask
		LPC_GPIO0->FIODIR  &= ~(1<<bit);	//set bit to output
		if(LPC_GPIO0->FIOPIN == 0) return 0;
		else return 1;
	}
	if(port == 1){							//port1
		LPC_GPIO1->FIOMASK =~(1<<bit);		//set mask
		LPC_GPIO1->FIODIR  &= ~(1<<bit);	//set bit to output
		if(LPC_GPIO1->FIOPIN == 0) return 0;
		else return 1;
	}
	if(port == 2){							//port2
		LPC_GPIO2->FIOMASK =~(1<<bit);		//set mask
		LPC_GPIO2->FIODIR  &= ~(1<<bit);	//set bit to output
		if(LPC_GPIO2->FIOPIN == 0) return 0;
		else return 1;
	}
	if(port == 3){							//port3
		LPC_GPIO3->FIOMASK =~(1<<bit);		//set mask
		LPC_GPIO3->FIODIR  &= ~(1<<bit);	//set bit to output
		if(LPC_GPIO3->FIOPIN == 0) return 0;
		else return 1;
	}
	return 0;
}

//Get a bit field from the selected register. The bit field is returned in the
//LSB position of the return value.
//offset: the location of the lowest bit in the field 0..31
//width: the width of the field 0..31 bits
unsigned int get_bits(unsigned int reg ,unsigned int offset, unsigned int width)
{
	// 1) shift right
	// 2) create mask
	// 3) clear unwanted bits
	// 4) return bit field

   return (reg >> offset) & ~(0xffffffff << width);
}

//Set a bit field.
//offset: the location of the lowest bit in the field 0..31
//width: the width of the field 0..31 bits
//value: the data to write to the field, located in the LSB position of the integer
void set_bits(volatile unsigned int *reg, unsigned int offset, unsigned int width, unsigned int value)
{
	uint32_t mask, temp;

   mask = ~((~(0xFFFFFFFF << width))<< offset);//  create the mask
   temp = *reg & mask;                         // clear the bit field
   *reg = temp | (value << offset);            // add the value we want
}

// Example:
// set_pin_function ( 0, 23, SECOND_FUNCTION);  // set pin P0_23 to analog input
void set_pin_function(unsigned int port, unsigned int pin,  unsigned int function)
{
    int reg, offset;

    offset = (pin%16)*2;  						//the offset is(2 bits/pin, 16 pins/register)

    // figure out which register to use
    reg = port * 2;								//first register
    if(pin>15) reg++;							//second register

    switch(reg){
        case 0:  set_bits(&LPC_PINCON->PINSEL0, offset, 2, function); break;
        case 1:  set_bits(&LPC_PINCON->PINSEL1, offset, 2, function); break;
        case 2:  set_bits(&LPC_PINCON->PINSEL2, offset, 2, function); break;
        case 3:  set_bits(&LPC_PINCON->PINSEL3, offset, 2, function); break;
        case 4:  set_bits(&LPC_PINCON->PINSEL4, offset, 2, function); break;
        case 5:  set_bits(&LPC_PINCON->PINSEL5, offset, 2, function); break;
        case 6:  set_bits(&LPC_PINCON->PINSEL6, offset, 2, function); break;
        case 7:  set_bits(&LPC_PINCON->PINSEL7, offset, 2, function); break;
        case 8:  set_bits(&LPC_PINCON->PINSEL8, offset, 2, function); break;
        case 9:  set_bits(&LPC_PINCON->PINSEL9, offset, 2, function); break;
    }
}

char GetHex(void)
{
	char digit;
	while(1){
		digit = get_char();
		if(digit >= '0' && digit <= '9'){
			return (digit-'0');
		}
		if(digit >= 'a' && digit <= 'f'){
			return (digit-'a'+10);
		}
		if(digit >= 'A' && digit <= 'F'){
			return (digit-'A'+10);
		}
	}
}

int ReadHex(void)
{
	return (GetHex()<<4 | GetHex());
}

//calibrated Feb 10 2012
float Reference_Angle(void)
{
int ADC;
	ADC = adcRead(0);
	if(ADC > g_ADC){
		return (float)(ADC-g_ADC)/g_ADC_P30;	//+17.36f;
	}else{
		return (float)(ADC-g_ADC)/g_ADC_M30;	//+14.49f;
	}
}

//
void TraceResults(float dt)
{
	dt /= 1000;									// ms to seconds
	g_print_rate = dt;
	LPC_SC->PCONP |= 1 << 22; 					// Power up Timer 2
	LPC_SC->PCLKSEL0 |= 1 << 2; 				// Clock for timer = CCLK
	LPC_TIM2->MR2 = (dt*1000000); 				// dt*1MHz match value
	LPC_TIM2->PR = 24;							// 72MHz/24/2 = 1MHz
	LPC_TIM2->MCR |= 1 << 6; 					// Interrupt on Match3
	LPC_TIM2->MCR |= 1 << 7; 					// Reset timer on Match3
	LPC_TIM2->TCR |= 1 << 7;					// Force Reset of Timer
	LPC_TIM2->TCR &= ~(1 << 7); 				// release reset on Timer
	NVIC_SetPriority(TIMER2_IRQn, 6);			// Set priority lower than I2C0
	NVIC_EnableIRQ(TIMER2_IRQn); 				// Enable timer interrupt
	LPC_TIM2->TCR |= 1 << 0; 					// Start timer

	g_dt_samples =0;							//debug sample timer
	print_index = 0;
	recording = ON;
}

void PrintTraceResults(void)
{
	int i;
	printf(",Reference,Roll,motor,Roll_Error,Roll_Error_Integral\n");
	for(i=0;i<1000;i++){
		debug1("%4.3f, %4.3f, ",i*g_print_rate,Trace0[i]);
		debug1("%4.3f, %4.3f, ",Trace1[i],Trace2[i]);
		debug1("%4.3f, %4.3f\n",Trace3[i],Trace4[i]);
		delay_ms(20);
	}
}

void TIMER2_IRQHandler(void)
{
    if((LPC_TIM2->IR & 1<<2)) { 				// if MR2 interrupt
    	if(recording){
    		Trace0[print_index]=Reference_Angle();
    		Trace1[print_index]=g_filter_roll;
    		Trace2[print_index]=g_motor_front;
    		Trace3[print_index]=g_PID_Roll_Error;
    		Trace4[print_index]=g_PID_Roll_Error_Integral;
			print_index++;
    	}
        LPC_TIM2->IR |= 1<<2; 					// Clear MR2 interrupt
    }
    gyro_saturation = 0;
}


